pub trait TransactionKind: Clone + Debug + Display + PartialEq + Eq + Hash + Serialize + DeserializeOwned + Send + Sync {
    // Required methods
    fn send() -> Self;
    fn receive() -> Self;
    fn mint() -> Self;
    fn freeze() -> Self;
    fn unfreeze() -> Self;
    fn unknown() -> Self;
}
The types of transactions supported by this network.

This trait represents an enum-like interface, with a variant for each transaction. Thus, the public interface consists of constructors for the CAP transaction type variants. The implementation may include additional variants for extensions to the protocol. It is required that two transaction kinds representing different kinds of transactiosn compare unequal. For example, TransactionKind::send() != TransactionKind::mint().

Required Methods

source

fn send() -> Self

source

fn receive() -> Self

source

fn mint() -> Self

source

fn freeze() -> Self

source

fn unfreeze() -> Self

source

fn unknown() -> Self

Implementors